import hashlib
import json
import datetime

def paste_id_for_source_and_key(conn, source, key):
    return conn.execute("""
        SELECT
            p.id
        FROM
            pastes AS p
        WHERE
            p.source_id = (
                SELECT
                    s2.id
                FROM
                    sources AS s2
                WHERE
                    s2.name = %s
                LIMIT 1
            ) AND
            p.key = %s
        LIMIT
            1
    """, (source, str(key)))


def insert_paste(conn, source, is_dox, key, full_url, text, syntax=None,
                 added_on=None, expires_on=None, user=None, title=None, scrape_url=None):

    hasher = hashlib.sha256()
    hasher.update(text.encode('utf8'))

    params = (
        source,
        bool(is_dox),
        title,
        str(key),
        full_url,
        scrape_url,
        text,
        hasher.hexdigest(),
        syntax,
        expires_on,
        user,
        added_on
    )

    return conn.execute("""
        INSERT INTO
            pastes AS p
            (
                source_id,
                is_dox,
                title,
                key,
                full_url,
                scrape_url,
                body,
                hash,
                syntax,
                expires_on,
                "user",
                added_on
            )
        VALUES
            (
                (
                    SELECT
                        s2.id
                    FROM
                        sources AS s2
                    WHERE
                        s2.name = %s
                    LIMIT
                        1
                ),
                %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s
            )
        RETURNING
            p.id;
    """, params)


def add_account_for_paste(conn, paste_id, network, account):
    return conn.execute("""
            INSERT INTO
                accounts AS a
                (
                    paste_id,
                    network_id,
                    name
                )
            VALUES
                (
                    %s,
                    (
                        SELECT
                            n2.id
                        FROM
                            networks AS n2
                        WHERE
                            n2.name = %s
                        LIMIT
                            1
                    ),
                    %s
                )
            RETURNING
                a.id
        """, (paste_id, network, account))


def add_crawl_for_account(conn, account_id, status, body):

    params = (
        account_id,
        account_id,
        status,
        body
    )

    return conn.execute("""
        INSERT INTO
            crawls AS c
            (
                account_id,
                delta,
                status_id,
                body
            )
        VALUES
            (
                %s,
                (
                    SELECT
                        COUNT(*)
                    FROM
                        crawls AS c2
                    WHERE
                        c2.account_id = %s
                ),
                (
                    SELECT
                        s2.id
                    FROM
                        statuses AS s2
                    WHERE
                        s2.name = %s
                    LIMIT
                        1
                ),
                %s
            )
        RETURNING
            c.id
    """, params)


def add_posts_to_crawl(conn, crawl_id, posts):

    post_params = []
    for a_post in posts:
        post_params.append(crawl_id)
        post_params.append(a_post)
    insert_queries = ",".join(["(%s, %s)"] * len(posts))

    return conn.execute("""
        INSERT INTO
            posts AS p
            (
                crawl_id,
                body
            )
        VALUES """ + insert_queries + """
        RETURNING
            p.id
    """, post_params)


def record_mentions_for_crawl(conn, crawl_id, mentions):

    mention_params = []
    for a_mention in mentions:
        mentioner = a_mention["user"]["screen_name"]
        mention_text = a_mention["text"]
        mention_raw = json.dumps(a_mention)

        mention_params.append(crawl_id)
        mention_params.append(mentioner)
        mention_params.append(mention_text)
        mention_params.append(mention_raw)
    insert_queries = ",".join(["(%s, %s, %s, %s)"] * len(mentions))

    return conn.execute("""
        INSERT INTO
            mentions AS m
            (
                crawl_id,
                text,
                commenter,
                metadata
            )
        VALUES """ + insert_queries + """
        RETURNING
            m.id
    """, mention_params)

def accounts_to_recrawl(conn, max_date, delta):

    return conn.execute("""
        SELECT
            c.account_id,
            a.name AS account_name,
            n.name AS network_name
        FROM
            crawls AS c
        JOIN
            accounts AS a ON (c.account_id = a.id)
        JOIN
            networks AS n ON (a.network_id = n.id)
        GROUP BY
            c.account_id,
            a.name,
            n.name
        HAVING
            MAX(c.created_on) <= %s AND
            MAX(c.delta) = (%s - 1)
    """, (max_date, delta))


def doxes_for_view_measure(conn):
    """Returns a list of pastebin keys, each describing a pastebin post that
    that should be scraped for how many views they've received.  The returned
    list will be a list of pastes that meet the following criteria:
        1. Classified as a dox OR has a paste id evenly divisible by 100
           (so that 0.01% of posts, dox-or-otherwise) will also have their
           view count tracked.
        2. Hasn't had its views measured in the last hour
        3. Was pasted to pastebin less than 2 weeks ago.

    Args:
        conn -- A psycopg2.connection object, that should be queried.

    Returns:
        A list of one or more strings, each describing a pastebin post that
        should be scraped for a view count.
    """
    query = """
    SELECT
        p.key
    FROM
        pastes as p
    LEFT JOIN
        (
            SELECT
                pv.paste_id,
                MAX(pv.created_on) AS created_on
            FROM
                pastes_views AS pv
            GROUP BY
                pv.paste_id
        ) AS pv ON (pv.paste_id = p.id)
    WHERE
        (
            p.is_dox = True OR
            p.id % 100 = 0
        ) AND
        (
            pv.created_on IS NULL OR
            pv.created_on < NOW() - interval '1 hour'
        ) AND
        p.source_id = 1 AND
        p.created_on > NOW() - interval '14 days'
    """
    return conn.execute(query)


def record_views_for_dox(conn, pastebin_key, num_views):
    """Records the number of times a pastebin paste have been viewed.

    Args:
        conn         -- A psycopg2.connection object, that should be queried.
        pastebin_key -- the unique identifier for a pastebin paste, as a str
        num_views    -- the number of times the given paste has been viewed.

    Returns:
        A future, wrapping the postgres insert query.
    """
    current_time = datetime.datetime.now()
    params = {
        "datetime": current_time,
        "num_views": num_views,
        "pastebin_key": pastebin_key,
    }
    query = """
        INSERT INTO
            pastes_views
            (
                paste_id,
                created_on,
                num_views,
                delta
            )
            VALUES
            (
                (
                    SELECT
                        p2.id
                    FROM
                        pastes AS p2
                    WHERE
                        p2.key = %(pastebin_key)s
                    LIMIT
                        1
                ),
                %(datetime)s,
                %(num_views)s,
                (
                    SELECT
                        COUNT(*)
                    FROM
                        pastes AS p2
                    JOIN
                        pastes_views AS pv2 ON (pv2.paste_id = p2.id)
                    WHERE
                        p2.key = %(pastebin_key)s
                )
            )
        RETURNING
            id
    """
    return conn.execute(query, params)
