import pprint
import datetime
import doxscrapers.sites.base
from doxscrapers.types import PostTeaser, Post


class PasteBinDoxScraper(doxscrapers.sites.base.BaseDoxScraper):


    def __init__(self, marshaller_host):
        super(PasteBinDoxScraper, self).__init__(1, marshaller_host, "pastebin")


    def items_in_index(self, proxy=None):
        """
        Fetches a list of PostTeaser objects, each representing a posting
        on PasteBin.

        Keyword Args:
            proxy -- An optional URL of a HTTP proxy, that should be used when
                     requesting against the API.

        Returns:
            None on an error, or a list of zero or more PostTeaser objects.
        """
        index_url = "https://pastebin.com/api_scraping.php?limit=100"
        api_response = self._json_for_url(index_url, proxy=proxy)
        if api_response is None:
            return None

        log_params = {
            u"count": len(api_response),
            u"url": index_url,
        }
        self._logger.info(u"Successfully received %(count)s items from %(url)s",
                          log_params)
        index_items = [PostTeaser(p["key"]) for p in api_response]
        return index_items


    def details_for_item(self, item, proxy=None):
        """
        Returns a Post object, describing the full contents of the post
        that was listed in an index of posts.

        Args:
            item -- a doxscrapers.types.PostTeaser object

        Keyword Args:
            proxy -- An optional URL of a HTTP proxy, that should be used when
                     requesting against the API.

        Returns:
            None on an error, and otherwise a doxscrapers.types.Post
            object.
        """
        self._throttler(self._req_pause)
        scrape_url = "http://pastebin.com/api_scrape_item.php?i={}".format(item.key)
        scrape_response = self._request_url(scrape_url, proxy=proxy)
        if scrape_response is None:
            return None
        post_text = scrape_response.text

        metadata_url = "http://pastebin.com/api_scrape_item_meta.php?i={}".format(item.key)
        metadata_response = self._json_for_url(metadata_url, proxy=proxy)

        if post_text is None or metadata_response is None:
            return None

        metadata = metadata_response[0]

        full_url = "http://pastebin.com/{}".format(item.key)

        post = Post(item.key, post_text, full_url)

        try:
            post.added_on = datetime.datetime.fromtimestamp(int(metadata['date']))
        except (ValueError, KeyError):
            log_params = {
                "response": pprint.pformat(metadata_response),
                "url": metadata_url,
            }
            self._logger.error("Invalid or missing 'date' value in JSON "
                               "response to url %(url)s: %(response)s",
                               log_params)
            return None

        if metadata['expire'] != "0":
            try:
                post.expires_on = datetime.datetime.fromtimestamp(int(metadata['expire']))
            except ValueError:
                log_params = {
                    "response": pprint.pformat(metadata_response),
                    "url": metadata_url,
                }
                self._logger.error("Invalid timestamp for 'expire' value in "
                                   "JSON in response to url %(url)s: "
                                   "%(response)s", log_params)
                return None

        if metadata['user'] != "":
            post.user = metadata['user']

        return post
