#!/usr/bin/env python

import argparse
import multiprocessing
import time
import logging
import functools
import doxpastebin.views
import doxpastebin.reporting
import doxpastebin.proxy

PARSER = argparse.ArgumentParser(description='Continuously poll for the number '
                                             'of times pastebin dox posts have '
                                             'been viewed.')
PARSER.add_argument('-i', '--ip', required=True,
                    help="The IP address of the db-frontend webserver.")
PARSER.add_argument('-p', '--port', required=True, type=int,
                    help="The port the db-frontend webserver.")
PARSER.add_argument('-l', '--logdir', default=None,
                    help="The directory to write logging information to.")
PARSER.add_argument('-v', '--verbose', action="store_true",
                    help="Whether to print out verbose informational "
                         "messages to STDOUT")
PARSER.add_argument('-s', '--secs', default=30, type=int,
                    help="Number of seconds to wait between re-polling the "
                         "pastebin or db-connector servers.")
PARSER.add_argument('-w', '--workers', default=None, required=False, type=int,
                    help="The maximum number of simultaneous requests that "
                         "should be made to pastebin.")
PARSER.add_argument('-x', '--proxy', default=None, action='append',
                    help="If provided, should be a 'host[:port]' value to "
                         "use when accessing the pastebin API.  If multiple "
                         "values are provided, they'll be rotated through.")
ARGS = PARSER.parse_args()

GET_PROXY = doxpastebin.proxy.make_rotator(ARGS.proxy)
doxpastebin.reporting.configure_logger(ARGS.logdir, ARGS.verbose)
LOGGER = logging.getLogger("dox-pastebin-views.log")

SLEEP_LOG_PARAMS = {"secs": ARGS.secs}
while True:
    UPDATE_VIEWS = False
    REQ_SUCCESS, REQ_KEYS = doxpastebin.views.pastebin_keys_to_measure(ARGS.ip, ARGS.port)
    if not REQ_SUCCESS:
        LOG_PARAMS = {"req_error": REQ_KEYS}
        LOGGER.error("Received error when fetching keys: %(req_error)s.", LOG_PARAMS)
        LOGGER.info("Sleeping for %(secs)i seconds, and then will try again.", SLEEP_LOG_PARAMS)
        time.sleep(ARGS.secs)
        continue

    if len(REQ_KEYS) == 0:
        LOGGER.info("Found no pastes for updating.  Sleeping for %(secs)i. "
                    "seconds, and then will try again.", SLEEP_LOG_PARAMS)
        time.sleep(ARGS.secs)
        continue

    POOL = multiprocessing.Pool(ARGS.workers)
    MANAGER = multiprocessing.Manager()
    SHARED_LIST = MANAGER.list([True])
    RECORD_KEY_FUNC = functools.partial(doxpastebin.views.record_proxied,
                                        ARGS.ip, ARGS.port, SHARED_LIST)
    WORK_RECORDS = [(K, GET_PROXY()) for K in REQ_KEYS[:30]]
    RESPONSES = POOL.map(RECORD_KEY_FUNC, WORK_RECORDS)
    for (IS_SUCCESS, RECORD_MESSAGE), PASTEBIN_KEY in zip(RESPONSES, REQ_KEYS):
        if IS_SUCCESS:
            UPDATE_VIEWS = True
            LOG_PARAMS = {
                "key": PASTEBIN_KEY,
                "views": RECORD_MESSAGE,
            }
            LOGGER.info("Successfully updated %(key)s with %(views)s views.",
                        LOG_PARAMS)
            continue

        # If this work request was never performed, because the process pool
        # was cancelled by another process in the pool, then no need to
        # record a message about it.
        if RECORD_MESSAGE is None:
            continue

        ERROR_LOG_PARAMS = {
            "key": PASTEBIN_KEY,
            "error": RECORD_MESSAGE,
        }
        LOGGER.error("Failed to record page views for %(key)s.  Error: %(error)s",
                     ERROR_LOG_PARAMS)
    POOL.close()

    if not UPDATE_VIEWS:
        LOGGER.info("Did not successfully update view counts for any paste, so "
                    "sleeping %(secs)i sec before continuing.", SLEEP_LOG_PARAMS)
    else:
        LOGGER.info("Finished recording view counts, looking for more pastes to record.")
    time.sleep(ARGS.secs)
